-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio           "EskaQuestTracker.Classes.QuestHeader"                      ""
-- ========================================================================== --
namespace "EQT"
-- ========================================================================== --
__DBTextOptions__( function() return _DB.Quest.header end, false)
class "QuestHeader" extend "IFrame" "IReusable"
  _QuestHeaderCache = setmetatable( {}, { __mode = "k" } )
  -- ======================================================================== --
  -- Handlers
  -- ======================================================================== --
  local function SetName(self, new, old, prop)
    local textTransform = QuestHeader.textTransform
    local txt = new

    if textTransform == "uppercase" then
      txt = txt:upper()
    elseif textTransform == "lowercase" then
      txt =  txt:lower()
    end
    self.frame.name:SetText(txt)
  end
  -- ======================================================================== --
  -- Methods                                                                  --
  -- ======================================================================== --
  __Arguments__{ Quest }
  function AddQuest(self, quest)
    if not self.quests:Contains(quest) then
      self.quests:Insert(quest)
      quest:SetParent(self.frame)
    end
  end

  __Arguments__{ Quest }
  function RemoveQuest(self, quest)
    self.quests:Remove(quest)
  end

  __Arguments__{}
  function GetQuestNum(self)
    return self.quests.Count
  end

  __Arguments__{}
  function Draw(self)
    local previousFrame
    local height = 0

    -- Quest compare function (Priorty : Distance > ID > Name)
    local function QuestSortMethod(a, b)
      if a.distance ~= b.distance then
        return a.distance < b.distance
      end

      if a.id ~= b.id then
        return a.id < b.id
      end
      return a.name < b.name
    end

    for index, quest in self.quests:Sort(QuestSortMethod):GetIterator() do
      --quest:ClearAllPoints()
      quest:Show()
      quest:Draw()
      if index == 1 then
        quest.frame:SetPoint("TOPLEFT", 0, -25)
        quest.frame:SetPoint("TOPRIGHT", 0, -25)
      else
        quest.frame:SetPoint("TOPLEFT", previousFrame, "BOTTOMLEFT", 0, -10)
        quest.frame:SetPoint("TOPRIGHT", previousFrame, "BOTTOMRIGHT")
      end
      previousFrame = quest.frame
      height = height + quest.height + 10
    end
    self.height = self.baseHeight + height
  end

  __Arguments__{}
  function Reset(self)
    self.name = nil
    self:ClearAllPoints()
    self:SetParent(nil)
    self:Hide()
  end

  __Arguments__{}
  function Refresh(self)
    local name = self.frame.name
    local font = _LibSharedMedia:Fetch("font", QuestHeader.textFont)
    local size = QuestHeader.textSize
    local color = QuestHeader.textColor
    local transform = QuestHeader.textTransform

    name:SetFont(font, size, "OUTLINE")
    name:SetTextColor(color.r, color.g, color.b)

    local txt = self.name
    if transform == "uppercase" then
      txt = txt:upper()
    elseif transform == "lowercase" then
      txt = txt:lower()
    end
    name:SetText(txt)
  end

  __Static__() function RefreshAll()
    for obj in pairs(_QuestHeaderCache) do
      obj:Refresh()
    end
  end

  -- ======================================================================== --
  -- Properties
  -- ======================================================================== --
  property "name" { TYPE = String, DEFAULT = "Misc", HANDLER = SetName }
  property "isActive" { TYPE = Boolean, DEFAULT = true }
  property "nearestQuestDistance" {
      GET = function(self)
        return self.quests:Sort("x,y=>x.distance<y.distance")[1].distance
      end
  }
  -- ======================================================================== --
  -- Contructor
  -- ======================================================================== --
  function QuestHeader(self, name)
    local frame = CreateFrame("Frame")
    --frame:SetBackdrop(_Backdrops.Common)
    --frame:SetBackdropColor(0, 1, 0, 1) -- 0.2
    --frame:SetBackdropBorderColor(0, 0, 0, 0)

    local name = frame:CreateFontString(nil, "OVERLAY")
    --local font = _LibSharedMedia:Fetch("font", QuestHeader.textFont)
    --local color = QuestHeader.textColor

    --name:SetFont(font, QuestHeader.textSize, "OUTLINE")
    --name:SetText(self.name)
    --name:SetTextColor(color.r, color.g, color.b)
    name:SetHeight(20)
    name:SetPoint("TOPLEFT", 10, 0)
    frame.name = name

    self.frame = frame
    self.height = 20
    self.baseHeight = self.height
    self.quests = ObjectArray(Quest)

    _QuestHeaderCache[self] = true

    self:Refresh()
  end

endclass "QuestHeader"
-- ========================================================================== --
-- == OnLoad Handler
-- ========================================================================== --
function OnLoad(self)
  _DB:SetDefault("Quest", {
    header = {
      textColor = { r = 1, g = 0.38, b = 0 },
      textSize = 9,
      textFont = "PT Sans Narrow Bold",
      textTransform = "uppercase", -- none, lowercase
    }
  })

  -- Register this class in the object manager
  _ObjectManager:Register(QuestHeader)
end
